@@ -17,7 +17,7 @@ def readfq(fp): # this is a generator function
                     last = l[:-1] # save this line
                     break
         if not last: break
-        name, seqs, last = last[1:].replace(" ", "_"), [], None
+        name, seqs, last = last[1:], [], None
         for l in fp: # read the sequence
             if l[0] in '@+>':
                 last = l[:-1]
